@@ -6,6 +6,7 @@
 #  id: a vector specifying cluster identifier
 #  Z: design matrix for the correlation model, should be all pairs j < k
 #    for each cluster
+#  link: a specification for the model link function
 #  maxiter: maximum number of iterations for Fisher scoring updates
 #  epsilon: tolerance for convergence. The default is 0.001
 #  printrange: print details of range violations. The default is TRUE
@@ -31,7 +32,7 @@
 #  niter: number of iterations in the Fisher scoring updates for model fitting
 
 
-continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
+continuous_maee <- function(y, X, id, Z, link, maxiter, epsilon, printrange,
                             alpadj, shrink, makevone) {
   # begin_end
   # Args:
@@ -64,10 +65,15 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  k: indicator for the mean
   #  X: covariate matrix for the cluster
   #  y: response vector for the cluster
+  #  link: a specification for the model link function
   # Returns:
   #  derivative of the empirical correlation over marginal mean covariates
-  get_corr_beta_deriv <- function(mu, gamma, j, k, X, y) {
-    row <- gamma * (X[j, ] / (y[j] - mu[j]) + X[k, ] / (y[k] - mu[k]))
+  get_corr_beta_deriv <- function(mu, gamma, j, k, X, y, link) {
+    beta_deriv_j <- create_beta_derivative(X[j, ], mu[j], link)
+    beta_deriv_k <- create_beta_derivative(X[k, ], mu[k], link)
+
+    row <- gamma * (beta_deriv_j / (y[j] - mu[j]) +
+      beta_deriv_k / (y[k] - mu[k]))
     return(row)
   }
 
@@ -81,6 +87,38 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
     return(y - mu)
   }
 
+  # marginal_mean_estimation creates mean estimation
+  # Args:
+  #  X: covariate matrix for cluster
+  #  beta: vector of marginal mean parameters
+  #  link: a specification for the model link function
+  # Returns:
+  #  marginal mean vector
+  marginal_mean_estimation <- function(X, beta, link) {
+    if (link == "identity") {
+      mu <- c(X %*% beta)
+    } else if (link == "log") {
+      mu <- exp(c(X %*% beta))
+    }
+    return(mu)
+  }
+
+
+  # create_beta_derivative creates derivative for beta estimating equation
+  # Args:
+  #  X: covariate matrix for cluster
+  #  mu: marginal mean vector
+  #  link: a specification for the model link function
+  # Returns:
+  #  derivative of beta estimating equation
+  create_beta_derivative <- function(X, mu, link) {
+    if (link == "identity") {
+      return(X)
+    } else if (link == "log") {
+      return(X * mu)
+    }
+  }
+
   # create_beta_covariance creates covariance for beta estimating equation
   # Args:
   #  phi: Dispersion parameter
@@ -117,6 +155,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  p: number of marginal mean parameters
   #  q: number of marginal correlation parameters
   #  phi: dispersion parameter
+  #  link: a specification for the model link function
   #  not_pos_def_alpadj_flag: checks if the alpha adjustment factor matrix is
   #                           positive definite
   # Returns:
@@ -124,7 +163,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  not_pos_def_alpadj_flag: checks if the alpha adjustment factor matrix is
   #                           positive definite
   phi_estimate <- function(Ustar, beta, alpha, y, X, Z, n, p, q,
-                           phi, not_pos_def_alpadj_flag) {
+                           phi, link, not_pos_def_alpadj_flag) {
     resid_sum_square <- 0
     naive_inv_prev <- ginv(Ustar[1:p, 1:p])
 
@@ -134,9 +173,9 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
     for (i in 1:length(n)) {
       X_c <- X[loc_x[i, 1]:loc_x[i, 2], , drop = FALSE]
       y_c <- y[loc_x[i, 1]:loc_x[i, 2]]
-      mu_c <- c(X_c %*% beta)
+      mu_c <- marginal_mean_estimation(X_c, beta, link)
 
-      beta_deriv <- X_c
+      beta_deriv <- create_beta_derivative(X_c, mu_c, link)
       beta_resid <- create_beta_residual(mu_c, y_c)
 
       if (loc_x[i, 1] == loc_x[i, 2]) {
@@ -187,6 +226,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  p: number of marginal mean parameters
   #  q: number of marginal correlation parameters
   #  phi: dispersion parameter
+  #  link: a specification for the model link function
   #  flag: performs an eigen-analysis of covariance to see if positive
   #        definite. only called when computing the variance at the
   #        end (0, 1). Prints warning for each cluster violation.
@@ -216,7 +256,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  not_pos_def_alpadj_flag: checks if the alpha adjustment factor matrix is
   #                           positive definite, terminates if not
   score_function <- function(Ustar_prev, beta, alpha, y, X, Z, n, p, q,
-                             phi, flag, range_flag, alpha_work_cov_flag,
+                             phi, link, flag, range_flag, alpha_work_cov_flag,
                              not_pos_def_work_cov_flag,
                              not_pos_def_alpadj_flag) {
     # score function
@@ -234,11 +274,11 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
 
       U_c <- rep(0, p + q)
       Ustar_c <- matrix(0, p + q, p + q)
-      mu_c <- c(X_c %*% beta)
+      mu_c <- marginal_mean_estimation(X_c, beta, link)
 
       # case when there was only 1 observation for this cluster
       if (loc_x[i, 1] == loc_x[i, 2]) {
-        beta_deriv <- X_c
+        beta_deriv <- create_beta_derivative(X_c, mu_c, link)
         beta_work_cov <- create_beta_covariance(phi, n[i])
         beta_resid <- create_beta_residual(mu_c, y_c)
 
@@ -258,7 +298,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
       # correlation means
       alpha_work_cov <- alpha_resid <- rep(0, choose(n[i], 2))
 
-      beta_deriv <- X_c
+      beta_deriv <- create_beta_derivative(X_c, mu_c, link)
       beta_work_cov <- create_beta_covariance(phi, n[i], gamma_c)
       beta_resid <- create_beta_residual(mu_c, y_c)
 
@@ -380,15 +420,35 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  y: the continuous outcome
   #  X: marginal mean covariates
   #  n: vector of cluster sample sizes
+  #  link: a specification for the model link function
   # Returns:
   #  beta: vector of marginal mean parameters
   #  Ustar: approximate information matrix
   #  phi: dispersion parameter
-  initial_beta <- function(y, X, n) {
-    beta <- solve(t(X) %*% X, t(X) %*% y)
-    u <- c(X %*% beta)
-    phi <- sum((y - u)^2) / (sum(n) - ncol(X))
-    Ustar <- t(X) %*% X / phi
+  initial_beta <- function(y, X, n, link) {
+    if (link == "identity") {
+      beta <- solve(t(X) %*% X, t(X) %*% y)
+      mu <- marginal_mean_estimation(X, beta, link)
+      beta_deriv <- create_beta_derivative(X, mu, link)
+      beta_resid <- create_beta_residual(mu, y)
+
+      phi <- sum(beta_resid^2) / (sum(n) - ncol(X))
+      Ustar <- t(X) %*% beta_deriv / phi
+    } else {
+      beta <- solve(t(X) %*% X, t(X) %*% y)
+
+      for (i in 1:2) {
+        mu <- marginal_mean_estimation(X, beta, link)
+        beta_resid <- create_beta_residual(mu, y)
+        beta_deriv <- create_beta_derivative(X, mu, link)
+        phi <- sum(beta_resid^2) / (sum(n) - ncol(X))
+
+        z <- t(X) %*% beta_resid
+        Ustar <- t(X) %*% beta_deriv / phi
+        d <- solve(Ustar, z)
+        beta <- beta + d
+      }
+    }
     return(list(beta = c(beta), Ustar = Ustar, phi = phi))
   }
 
@@ -433,6 +493,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  p: number of marginal mean parameters
   #  q: number of marginal correlation parameters
   #  phi: dispersion parameter
+  #  link: a specification for the model link function
   #  alpha_work_cov_flag: checks if all variances for alpha estimating
   #                       equations are positive, terminates if not
   #  not_pos_var_est_flag: checks if any robust covariance matrix
@@ -452,14 +513,16 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  not_pos_def_alpadj_flag: checks if the alpha adjustment factor matrix is
   #                           positive definite, terminates if not
   variance_estimator <- function(Ustar_prev, beta, alpha, y, X, Z, n, p, q,
-                                 phi, alpha_work_cov_flag, not_pos_var_est_flag,
+                                 phi, link, alpha_work_cov_flag,
+                                 not_pos_var_est_flag,
                                  not_pos_def_work_cov_flag,
                                  not_pos_def_alpadj_flag) {
     score_res <- score_function(Ustar_prev, beta, alpha, y, X, Z, n, p,
-      q, phi,
+      q, phi, link,
       flag = 1, range_flag = 0, alpha_work_cov_flag,
       not_pos_def_work_cov_flag, not_pos_def_alpadj_flag
     )
+
     U <- score_res$U
     UUtran <- score_res$UUtran
     Ustar <- score_res$Ustar
@@ -504,13 +567,13 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
     for (i in 1:length(n)) {
       X_c <- X[loc_x[i, 1]:loc_x[i, 2], , drop = FALSE]
       y_c <- y[loc_x[i, 1]:loc_x[i, 2]]
-      mu_c <- c(X_c %*% beta)
+      mu_c <- marginal_mean_estimation(X_c, beta, link)
       U_i <- U_c <- rep(0, p + q)
       Ustar_c <- matrix(0, p + q, p + q)
 
       # case when there was only 1 observation for this cluster
       if (loc_x[i, 1] == loc_x[i, 2]) {
-        beta_deriv <- X_c
+        beta_deriv <- create_beta_derivative(X_c, mu_c, link)
         beta_work_cov <- create_beta_covariance(mu_c, n[i])
         beta_resid <- create_beta_residual(mu_c, y_c)
 
@@ -545,7 +608,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
       gamma_c <- c(Z_c %*% alpha)
 
       # commands for beta
-      beta_deriv <- X_c
+      beta_deriv <- create_beta_derivative(X_c, mu_c, link)
       beta_work_cov <- create_beta_covariance(phi, n[i], gamma_c)
       beta_resid <- create_beta_residual(mu_c, y_c)
 
@@ -597,8 +660,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
             alpha_work_cov[l] <- 1 + gamma_c[l]^2
           }
           corr_beta_deriv[l, ] <- get_corr_beta_deriv(
-            mu_c, gamma_c[l], j, k, X_c,
-            y_c
+            mu_c, gamma_c[l], j, k, X_c, y_c, link
           )
 
           # Matrix-based multiplicative correction, (I - H_i)^{-1}
@@ -713,6 +775,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #  X: marginal mean covariates
   #  Z: marginal correlation covariates
   #  n: vector of cluster sample sizes
+  #  link: a specification for the model link function
   #  maxiter: max number of iterations
   #  epsilon: tolerence for convergence
   #  alpha_work_cov_flag: algorithm terminated due to non-positive variance
@@ -749,7 +812,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   #                             definite, terminates if not
   #  not_pos_def_alpadj_flag: checks if the alpha adjustment factor matrix
   #                           is positive definite, terminates if not
-  model_fit <- function(y, X, Z, n, maxiter, epsilon, alpha_work_cov_flag,
+  model_fit <- function(y, X, Z, n, link, maxiter, epsilon, alpha_work_cov_flag,
                         singular_naive_var_flag, not_pos_var_est_flag,
                         alpha_shrink_flag, not_pos_def_work_cov_flag,
                         not_pos_def_alpadj_flag) {
@@ -761,7 +824,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
 
     range_flag <- 0
     alpha <- rep(0.01, q)
-    init_res <- initial_beta(y, X, n)
+    init_res <- initial_beta(y, X, n, link)
     beta <- init_res$beta
     Ustar <- init_res$Ustar
     phi <- init_res$phi
@@ -778,7 +841,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
         not_pos_def_alpadj_flag <- 0
 
         score_res <- score_function(Ustar_prev, beta, alpha, y, X, Z,
-          n, p, q, phi,
+          n, p, q, phi, link,
           flag = 0, range_flag, alpha_work_cov_flag,
           not_pos_def_work_cov_flag,
           not_pos_def_alpadj_flag
@@ -839,7 +902,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
         alpha <- theta[(p + 1):(p + q)]
         phi_res <- phi_estimate(
           Ustar, beta, alpha, y, X, Z, n, p,
-          q, phi, not_pos_def_alpadj_flag
+          q, phi, link, not_pos_def_alpadj_flag
         )
         phi <- phi_res$phi
         converge <- (max(abs(delta)) <= epsilon)
@@ -854,7 +917,7 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
     # inference
     var_res <- variance_estimator(
       Ustar_prev, beta, alpha, y, X, Z, n,
-      p, q, phi, alpha_work_cov_flag,
+      p, q, phi, link, alpha_work_cov_flag,
       not_pos_var_est_flag,
       not_pos_def_work_cov_flag,
       not_pos_def_alpadj_flag
@@ -951,7 +1014,6 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   # reasons for non-results are identified and tallied
   alpha_work_cov_flag <- 0
   singular_naive_var_flag <- 0
-  converge_flag <- 0 # omitted in the arguments for model fit
   not_pos_var_est_flag <- 0
   alpha_shrink_flag <- 0
   not_pos_def_work_cov_flag <- 0
@@ -964,9 +1026,14 @@ continuous_maee <- function(y, X, id, Z, maxiter, epsilon, printrange,
   id <- id1
   n <- as.vector(table(id))
 
+  # link default
+  if (is.null(link)) {
+    link <- "identity"
+  }
+
   # fit the GEE2 model
   model_fit_res <- model_fit(
-    y, X, Z, n, maxiter, epsilon, alpha_work_cov_flag,
+    y, X, Z, n, link, maxiter, epsilon, alpha_work_cov_flag,
     singular_naive_var_flag, not_pos_var_est_flag, alpha_shrink_flag,
     not_pos_def_work_cov_flag, not_pos_def_alpadj_flag
   )
